package com.jiuqi.db.impl;

import java.util.List;
import com.jiuqi.db.DBFunction;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import com.jiuqi.db.DBMetaData;

public class MySQLDBMetaData implements DBMetaData
{
    private static final String LeftParenthesis = "(";
    private static final String RightParenthesis = ")";
    private static final String comma = ",";
    private static final String IsNull = "NULL";
    private static final String NotNull = "NOT NULL";
    private static final String space = " ";
    private static final String DEF = " DEFAULT ";
    private static final String BIT = "BIT";
    private static final String TINYINT = "TINYINT";
    private static final String SMALLINT = "SMALLINT";
    private static final String INTEGER = "INT";
    private static final String BIGINT = "BIGINT";
    private static final String FLOAT = "FLOAT";
    private static final String REAL = "REAL";
    private static final String DOUBLE = "DOUBLE";
    private static final String NUMERIC = "DECIMAL";
    private static final String DECIMAL = "DECIMAL";
    private static final String CHAR = "CHAR";
    private static final String VARCHAR = "VARCHAR";
    private static final String LONGVARCHAR = "MEDIUMTEXT";
    private static final String TIMESTAMP = "DATETIME";
    private static final String VARBINARY = "VARBINARY";
    private static final String LONGVARBINARY = "MEDIUMBLOB";
    private static final String BLOB = "BLOB";
    private static final String CLOB = "TEXT";
    private int maxColumnCount;
    private int maxRecordSize;
    
    public MySQLDBMetaData(final DatabaseMetaData metaData) throws SQLException {
        this.maxColumnCount = metaData.getMaxColumnsInTable();
        this.maxRecordSize = metaData.getMaxRowSize();
    }
    
    private void raise(final String msg) throws SQLException {
        throw new SQLException(msg);
    }
    
    public DBFunction createDBFunction() {
        return new MySQLDBFunction();
    }
    
    public int databaseType() {
        return 5;
    }
    
    public boolean getDDLCausesTransactionCommit() {
        return true;
    }
    
    public boolean getDDLIgnoredInTransactions() {
        return false;
    }
    
    public String getDefaultScheme() {
        return null;
    }
    
    public String getDropFieldDDL(final String tableName, final String fieldName) {
        return "ALTER TABLE " + tableName + " DROP  " + fieldName;
    }
    
    public String getDropFieldPreSql(final String tableName, final String fieldName) {
        return null;
    }
    
    public String getFieldDDL(final int sqlType, final int size, final int scale, final boolean isNull, final String defValue) throws SQLException {
        final StringBuffer result = new StringBuffer();
        switch (sqlType) {
            case -7: {
                result.append("BIT");
                break;
            }
            case -6: {
                result.append("TINYINT");
                break;
            }
            case 5: {
                result.append("SMALLINT");
                break;
            }
            case 4: {
                result.append("INT");
                break;
            }
            case -5: {
                result.append("BIGINT");
                break;
            }
            case 6: {
                result.append("FLOAT");
                break;
            }
            case 7: {
                result.append("REAL");
                break;
            }
            case 8: {
                result.append("DOUBLE");
                break;
            }
            case 2: {
                if (size < 0 || size > 65 || scale < 0 || scale > 65) {
                    this.raise("精度或小数位定义错误 精度:" + size + "小数位:" + scale);
                }
                result.append("DECIMAL");
                result.append("(");
                result.append(size);
                result.append(",");
                result.append(scale);
                result.append(")");
                break;
            }
            case 3: {
                if (size < 0 || size > 65 || scale < 0 || scale > 65) {
                    this.raise("精度或小数位定义错误 精度:" + size + "小数位:" + scale);
                }
                result.append("DECIMAL");
                result.append("(");
                result.append(size);
                result.append(",");
                result.append(scale);
                result.append(")");
                break;
            }
            case 1: {
                if (size > 255 || size < 0) {
                    this.raise("长度定义错误 应在0~255之间");
                }
                result.append("CHAR");
                result.append("(");
                result.append(size);
                result.append(")");
                break;
            }
            case 12: {
                if (size > 65535 || size < 0) {
                    this.raise("长度定义错误 应在0~65535之间");
                }
                result.append("VARCHAR");
                result.append("(");
                result.append(size);
                result.append(")");
                break;
            }
            case -1: {
                result.append("MEDIUMTEXT");
                break;
            }
            case 91:
            case 92:
            case 93: {
                result.append("DATETIME");
                break;
            }
            case -3:
            case -2: {
                if (size > 255 || size < 0) {
                    this.raise("长度定义错误 应在0~255之间");
                }
                result.append("VARBINARY");
                result.append("(");
                result.append(size);
                result.append(")");
                break;
            }
            case -4: {
                result.append("MEDIUMBLOB");
                break;
            }
            case 2004: {
                result.append("BLOB");
                break;
            }
            case 2005: {
                result.append("TEXT");
                break;
            }
        }
        if (result.length() == 0) {
            this.raise("不支持的数据类型" + sqlType);
        }
        result.append(" ");
        result.append(isNull ? "NULL" : "NOT NULL");
        result.append(" ");
        if (defValue != null && !defValue.equals("")) {
            result.append(" DEFAULT ");
            result.append(defValue);
        }
        return result.toString();
    }
    
    public boolean getModifyFieldSqls(final String tableName, final String sFieldName, final int sSqlType, final int sSize, final int sScale, final boolean sIsNull, final String sDefValue, final String dFieldName, final int dSqlType, final int dSize, final int dScale, final boolean dIsNull, final String dDefValue, final List sqls) throws SQLException {
        final StringBuffer sql = new StringBuffer();
        if (sFieldName.equals(dFieldName)) {
            sql.append("ALTER TABLE ");
            sql.append(tableName);
            sql.append(" CHANGE ");
            sql.append(sFieldName);
            sql.append(" ");
            sql.append(dFieldName);
            sql.append(" ");
            sql.append(this.getFieldDDL(dSqlType, dSize, dScale, dIsNull, dDefValue));
            sqls.add(sql.toString());
            return true;
        }
        return false;
    }
    
    public boolean getSupportDropColumn() {
        return true;
    }
    
    public boolean getTruncateNumber() {
        return false;
    }
    
    public int maxColumnByteSize(final int sqlType) throws SQLException {
        int result = 0;
        switch (sqlType) {
            case -7: {
                result = 64;
                break;
            }
            case -6: {
                result = 8;
                break;
            }
            case 5: {
                result = 16;
                break;
            }
            case 4: {
                result = 32;
                break;
            }
            case -5: {
                result = 64;
                break;
            }
            case 6: {
                result = 24;
                break;
            }
            case 7: {
                result = 4;
                break;
            }
            case 2: {
                result = 65;
                break;
            }
            case 3: {
                result = 65;
                break;
            }
            case 1: {
                result = 255;
                break;
            }
            case 12: {
                result = 65535;
                break;
            }
            case -1: {
                result = 16;
                break;
            }
            case 93: {
                result = 19;
                break;
            }
            case -2: {
                result = -1;
                break;
            }
            case -3: {
                result = -1;
                break;
            }
            case -4: {
                result = -1;
                break;
            }
            case 2004: {
                result = 16;
                break;
            }
            case 2005: {
                result = 16;
                break;
            }
        }
        if (result == 0) {
            this.raise("不支持的数据类型 " + Integer.toString(sqlType));
        }
        return result;
    }
    
    public int maxColumnCount() {
        return this.maxColumnCount;
    }
    
    public int maxFieldNameLength() {
        return 64;
    }
    
    public int maxIndexNameLength() {
        return 64;
    }
    
    public int maxRecordSize() {
        return this.maxRecordSize;
    }
    
    public int maxTableNameLength() {
        return 64;
    }
}
